// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import os.log

public extension UIApplication {
    /// Calls `phoneNumber` by using the tel:// scheme.
     func callPhoneNumber(phoneNumber: String) {
         self.openSafe(url: "tel://\(phoneNumber)", options: nil, completionHandler: nil)
     }


     /// Check whether the current device supports the tel:// scheme
     func canOpenPhoneUrl() -> Bool {
         let canOpen: Bool

         if let phoneTestUrl: URL = URL(string: "tel://") {
             canOpen = self.canOpenURL(phoneTestUrl)
         }
         else {
             canOpen = false
         }

         return canOpen
     }

    /// Opens `address` in Apple Maps.
     func openAppleMaps(address: String) {
         self.openSafe(url: "https://maps.apple.com/?q=\(address.urlEncoded)", options: nil, completionHandler: nil)
     }

     /// Opens `address` in Google maps if available. If `useAppleMapsIfUnable` is `true` the function will fallback to Apple Maps, if Google Maps is not available. Otherwise the function will do nothing.
     func openGoogleMaps(address: String, useAppleMapsIfUnable: Bool = false) {
         openSafe(url: "comgooglemaps://?q=\(address.urlEncoded)", options: nil) { [weak self] (success: Bool) -> Void in
             if !success && useAppleMapsIfUnable {
                 self?.openAppleMaps(address: address)
             }
         }
     }

    /// Opens a url safe by validating the `url` parameter as a valid URL-object and asks the application if it can open the `url`.
    ///
    /// If the `canOpenUrl` check fails the method will do nothing.
    func openSafe(url: String, options: [String: Any]?, completionHandler: ((Bool) -> Void)?) {
        if let safeUrl = URL(string: url) {
            self.openSafe(url: safeUrl, options: nil, completionHandler: completionHandler)
        } else {
            completionHandler?(false)
            #warning("Replace with correct bundle id when they are available")
            os_log("Could not create link, url is not valid, was: @%", log: OSLog(subsystem: "com.trifork.mingraviditet.appleextensions", category: "AppleExtensions"), url)
        }
    }
    
    /// Opens a url safe by asking the application if it can open the `url`.
    ///
    /// If the `canOpenUrl` check fails the method will do nothing.
    func openSafe(url: URL, options: [UIApplication.OpenExternalURLOptionsKey: Any]?, completionHandler: ((Bool) -> Void)?) {
        if !self.canOpenURL(url) {
            #warning("Replace with correct bundle id when they are available")
            os_log("Could not open url: @%", log: OSLog(subsystem: "com.trifork.mingraviditet.appleextensions", category: "AppleExtensions"), url.absoluteString)
            completionHandler?(false)
        }
        if #available(iOS 10.0, *) {
            open(url, options: options ?? [:], completionHandler: completionHandler)
        } else {
            completionHandler?(openURL(url))
        }
    }
    
    /// Recursively searches for the top most view controller based on a view controller as starting point.
    ///
    /// 1. Checks for UINavigationController's visibleViewController
    /// 2. Checks for selected UITabBarController's selectedViewController
    /// 3. Check for presented view controllers.
    static func topViewController(controller: UIViewController? = UIApplication.shared.keyWindow?.rootViewController) -> UIViewController? {
        if let navigationController = controller as? UINavigationController {
            return topViewController(controller: navigationController.visibleViewController)
        }
        if let tabController = controller as? UITabBarController {
            if let selected = tabController.selectedViewController {
                return topViewController(controller: selected)
            }
        }
        if let presented = controller?.presentedViewController {
            return topViewController(controller: presented)
        }
        return controller
    }
}
