// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine

public protocol NetworkSession {
    /// Returns a publisher that wraps a URL session data task for a given URL request.
    ///
    /// The publisher publishes data when the task completes, or terminates if the task fails with an error.
    /// - Parameter request: The URL request for which to create a data task.
    /// - Returns: A publisher that wraps a data task for the URL request.
    @available(macOS 10.15, iOS 13.0, tvOS 13.0, watchOS 6.0, *)
    func dataTaskPublisher(for request: URLRequest) -> URLSession.DataTaskPublisher
}

// Helper mehtods
public extension NetworkSession {
    /// Returns a publisher that wraps a URL session data task for a given URL request.
    ///
    /// The publisher publishes data when the task completes, or terminates if the task fails with an error.
    /// - Parameter request: The HTTP request for which to create a data task.
    /// - Returns: A publisher that wraps a data task for the URL request.
    @available(macOS 10.15, iOS 13.0, tvOS 13.0, watchOS 6.0, *)
    func dataTaskPublisher<Request: HTTPRequest>(for request: Request) -> AnyPublisher<Request.ResponseType, Error> {
        let urlRequest = createRequest(with: request)

        return dataTaskPublisher(for: urlRequest)
            .tryMap { data, response in
                try Request.ResponseType.init(input: data, response: response as? HTTPURLResponse)
        }
        .mapError { $0 }
        .eraseToAnyPublisher()
    }
}

private extension NetworkSession {
    private func createRequest<Request: HTTPRequest>(with request: Request) -> URLRequest {
        var url = request.url

        request.query?.forEach { key, value in
            url = url.appendingQueryComponent(key, value: value)
        }

        var urlRequest = URLRequest(url: url)

        urlRequest.httpMethod = request.method.rawValue.uppercased()
        urlRequest.httpBody = request.body

        request.headers?.forEach({ key, value in
            urlRequest.addValue(value, forHTTPHeaderField: key)
        })

        if TriforkSwiftNetworkingConfig.isDebugPrintingEnabled {
            log(for: urlRequest)
        }

        return urlRequest
    }

    private func log(for urlRequest: URLRequest) {
        let bodyString: String
        let noBodyDataString = "<NO BODY DATA>"
        if let data = urlRequest.httpBody {
            bodyString = String(data: data, encoding: .utf8) ?? noBodyDataString
        } else {
            bodyString = noBodyDataString
        }
        let logMessage =  """
Created \(urlRequest.httpMethod ?? "<UNKNOWN>") request for:
URL: \(urlRequest.url?.absoluteString ?? "<UNKNOWN>")
HEADERS ⬇️:
\(urlRequest.allHTTPHeaderFields?.description ?? "<NO HEADERS>")
BODY ⬇️:
\(bodyString)
"""
        Logger.log(logMessage)
    }
}
