// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import TIM
import DependencyInjection
import Logger
import UIKit
import Combine
import LexiconServices
import ChecklistServices
import LocalAuthentication
import ThePerfectApp
import HTTPNetworking
import SharedServices
import UserSettingsServices
import MaternityServices
import CMSServices

enum BuildConfiguration: String {
    case debug = "Debug"
    case development = "Development"
    case test = "Test"
    case testMock = "TestMock"
    case release = "Release"
    case unknown = "Unknown"
}

struct Configuration {
    let backendUrl: String
    let useMock: Bool
    let loggerConfig: OSLoggerConfig
    let tim: TIMConfiguration
}

final class App {
    static let buildConfiguration = BuildConfiguration(rawValue: Bundle.main.buildConfiguration) ?? .unknown
    static let versionString: String = Bundle.main.versionString
    static let configuration = Configuration.build(for: buildConfiguration)
    static let isTesting: Bool = ProcessInfo.processInfo.environment["IS_TESTING"] == "1"

    private static let multiLogger = MultiLogger(loggers: [])

    /// Used to present OpenID Connect login view
    static var windowRootViewController: UIViewController? {
        UIApplication.topViewController()
    }

    static func buildDependencies() {
        setupThePerfectApp()
        setupLoggers()
        setupTriforkIdentityManager()

        let apiConfiguration = ApiConfiguration()

        Resolver.shared.registerAsSingleton(Logger.self, multiLogger)
        Resolver.shared.registerAsSingleton(TIMDataStorage.self, TIM.storage)
        Resolver.shared.registerAsSingleton(TIMAuth.self, TIM.auth)
        Resolver.shared.register(LocalAuthenticationContext.self, { LAContext() })
        Resolver.shared.registerAsSingleton(LexiconApiConfiguration.self, apiConfiguration)
        Resolver.shared.registerAsSingleton(ChecklistApiConfiguration.self, apiConfiguration)
        Resolver.shared.registerAsSingleton(MaternityApiConfiguration.self, apiConfiguration)
        Resolver.shared.registerAsSingleton(NetworkSession.self, URLSession.shared)
        Resolver.shared.registerAsSingleton(TokenService.self, TokenServiceImpl())
        Resolver.shared.registerAsSingleton(UserSettingsService.self, UserSettingsServiceImpl())
        Resolver.shared.registerAsSingleton(UserContextProtocol.self, UserContext())
        Resolver.shared.registerAsSingleton(CarePlanService.self, CarePlanServiceImpl())

        #warning("Dictionary uses mock until we know more about the CMS interface.")
        Resolver.shared.registerAsSingleton(DictionaryService.self, MockDictionaryService())

        if configuration.useMock {
            addMockServices()
        } else {
            addServices()
        }
        addSpecialViews()
    }

    private static func setupLoggers() {
        var loggers: [LoggerProtocol] = [OSLogger(config: configuration.loggerConfig)]
        if buildConfiguration == .debug {
            TriforkSwiftNetworkingConfig.isDebugPrintingEnabled = false // Can be enabled when debugging API requests
            loggers.append(ErrorFlashLogger())
        } else {
            // TPA should only log when not debugging
            loggers.append(ThePerfectAppLogger())
        }
        multiLogger.loggers = loggers
    }

    private static func addServices() {
        Resolver.shared.registerAsSingleton(LexiconService.self, LexiconServiceImpl())
        Resolver.shared.registerAsSingleton(ChecklistService.self, ChecklistServiceImpl())
    }

    private static func addMockServices() {
        Resolver.shared.registerAsSingleton(LexiconService.self, MockLexiconService())
        Resolver.shared.registerAsSingleton(ChecklistService.self, MockChecklistService())

        if buildConfiguration == .debug {
            MockFlags.failRisk = .small
        } else {
            MockFlags.failRisk = .disabled
        }
    }

    private static func addSpecialViews() {
        // The activity monitor view is not added in debug, to support view debugging (crashes when added)
        // This means that you will never see a login timeout during debugging
        // The timeout is set for one hour when debugging to avoid this.
        if buildConfiguration == .debug {
            Resolver.shared.registerAsSingleton(ActivityMonitor.self, DebugActivityMonitorWindow())
        } else {
            Resolver.shared.registerAsSingleton(ActivityMonitor.self, ActivityMonitorWindow())
        }
    }

    private static func setupTriforkIdentityManager() {
        TIM.configure(configuration: configuration.tim)
    }

    private static func setupThePerfectApp() {
        switch App.buildConfiguration {
        case .debug, .release:
            TPA.shared().loggingDestinations = [] // none
        case .testMock,
             .test:
            TPA.shared().loggingDestinations = [.remote]
            TPA.shared().remoteLogLevel = .info
            TPA.shared().updateNotification = .automatic
            TPA.shared().feedbackInvocation = .eventShake
        default:
            break
        }

        TPA.shared().isAnalyticsEnabled = true
        TPA.shared().crashReporting = .alwaysSend
        TPA.shared().isNonFatalEnabled = true
        TPA.shared().start(baseUrl:"https://insp.tpa.io/", projectUuid: "39b64243-828e-45a7-a4f1-fd70335961bd")
    }
}

private extension Configuration {
    //swiftlint:disable:next function_body_length
    static func build(for buildConfiguration: BuildConfiguration) -> Configuration {
        let timConfiguration = TIMConfiguration.forBuildConfiguration(buildConfiguration)

        switch buildConfiguration {
        case .debug:
            return Configuration(
                backendUrl: "",
                useMock: false,
                loggerConfig: OSLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .debug,
                    isDevelopmentInfoEnabled: false,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .test,
             .development:
            return Configuration(
                backendUrl: "",
                useMock: false,
                loggerConfig: OSLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .info,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .testMock:
            return Configuration(
                backendUrl: "",
                useMock: true,
                loggerConfig: OSLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .info,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .release:
            return Configuration(
                backendUrl: "",
                useMock: false,
                loggerConfig: OSLoggerConfig(
                    defaultCategory: buildConfiguration.rawValue,
                    minimumLogLevel: .error,
                    isDevelopmentInfoEnabled: true,
                    isEmojisEnabled: true
                ),
                tim: timConfiguration
            )
        case .unknown:
            fatalError("App was started with unknown build configuration 🤯")
        }
    }
}
