// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI
import TIM
import CMSServices

final class SceneDelegate: UIResponder, UIWindowSceneDelegate {

    var window: UIWindow?

    private var cancelBag = CancelBag()

    @DI private var logger: Logger
    @DI private var timDataStorage: TIMDataStorage
    @DI private var timAuth: TIMAuth
    @DI private var activityMonitor: ActivityMonitor
    @DI private var userContext: UserContextProtocol
    @DI private var dictionaryService: DictionaryService

    func scene(_ scene: UIScene, willConnectTo session: UISceneSession, options connectionOptions: UIScene.ConnectionOptions) {
        guard let windowScene = scene as? UIWindowScene else { return }

        // Swizzle `UIColor.systemBackground` to change the default background color system-wide.
        // We've have tried to solve this in several different ways, but this is the only way, that
        // doesn't ruins other UI behaviours while setting the background (`UIHostingController` is the big scapegoat in this case 🐐)
        UIColor.swizzleSystemBackground()
        
        let window = UIWindow(windowScene: windowScene)
        self.window = window

        listenForNotifications()

        if !userContext.hasHandledOnboardingFlow {
            // User hasn't finished onboarding
            timDataStorage.clearAllUsers() // Clean up of all users, if onboarding haven't been completed yet
            let onboarding = UIHostingController(rootView: OnboardingView())
            changeRootViewController(onboarding, animated: false)
        } else if userContext.hasHandledOnboardingFlow && !userContext.hasUserRegistered {
            // User has finished onboarding, but skipped login
            self.handleNotification(.showTabBarController, animated: false)
        } else if userContext.hasHandledOnboardingFlow && userContext.hasUserRegistered {
            // User has finished onboarding and registered with TIM
            changeRootViewController(UIHostingController(rootView: SessionLoadingView()), animated: false)

            timAuth.accessToken()
                .sink(receiveCompletion: { completion in
                    switch completion {
                    case .failure(let error):
                        self.logger.info("Failed to receive access token, the session must be invalid: \(error.localizedDescription)")
                        if let userId = self.timDataStorage.availableUserIds.first {
                            self.handleNotification(.showLogin(userId: userId), animated: false)
                        } else {
                            self.logger.critical("This state should never happen, but we are making a fallback to onboarding if it does happen!")
                            self.handleNotification(.resetToOnboarding, animated: false)
                        }
                    case .finished:
                        break
                    }
                }, receiveValue: { at in
                    guard at.userId != nil else {
                        self.logger.critical("Access Token did not have a valid userId. Clearing all users.")
                        self.timDataStorage.clearAllUsers()
                        self.handleNotification(.resetToOnboarding, animated: false)
                        return
                    }
                    self.logger.debug("Access Token:\n\(at)")
                    self.handleNotification(.showTabBarController, animated: false)
                })
                .store(in: &cancelBag)
        }
        window.makeKeyAndVisible()
    }

    func sceneDidDisconnect(_ scene: UIScene) {
        // Called as the scene is being released by the system.
        // This occurs shortly after the scene enters the background, or when its session is discarded.
        // Release any resources associated with this scene that can be re-created the next time the scene connects.
        // The scene may re-connect later, as its session was not necessarily discarded (see `application:didDiscardSceneSessions` instead).
    }

    func sceneDidBecomeActive(_ scene: UIScene) {
        SecurityCoverView.shared.detachFromWindow()

        #warning("This might not be the right way to do this! Revisit when we know more!")
        // Update stored dictionary data.
        dictionaryService.fetchDictionary()
            .sink(receiveCompletion: { _ in }, receiveValue: { _ in })
            .store(in: &cancelBag)

        if !App.isTesting {
            let shouldLockForTimeout: Bool = activityMonitor.shouldRequireLoginOnAppActive && timAuth.isLoggedIn
            let currentUserId = timAuth.refreshToken?.userId ?? timDataStorage.availableUserIds.first

            if let userId = currentUserId {
                if shouldLockForTimeout {
                    handleNotification(.showLogin(userId: userId), animated: true)
                } else if timAuth.isLoggedIn {
                    activityMonitor.startTracking()
                }
            }
        }
    }

    func sceneWillResignActive(_ scene: UIScene) {
        activityMonitor.appBecameInactive()
        activityMonitor.stopTracking()
    }

    func sceneWillEnterForeground(_ scene: UIScene) {
        // Called as the scene transitions from the background to the foreground.
        // Use this method to undo the changes made on entering the background.
    }

    func sceneDidEnterBackground(_ scene: UIScene) {
        SecurityCoverView.shared.attachToWindow()
    }

    func scene(_ scene: UIScene, openURLContexts URLContexts: Set<UIOpenURLContext>) {
        for url: URL in URLContexts.map({ $0.url }) {
            timAuth.handleRedirect(url: url)
        }
    }

    // MARK: -
    private func listenForNotifications() {
        NotificationCenter.default
            .listen(for: Notification.Name.SceneDelegate.changeRootViewController)
            .sink(receiveValue: { (message: NotificationMessage) in
                self.handleNotification(message, animated: true)
            })
            .store(in: &cancelBag)
    }

    private func changeRootViewController(_ rootViewController: UIViewController, animated: Bool) {
        guard let window = window else {
            fatalError("Failed to change rootViewController, since the windows was nil")
        }
        rootViewController.view.backgroundColor = R.color.viewBackground()
        window.rootViewController = rootViewController
        window.makeKeyAndVisible()

        if animated {
            UIView.transition(
                with: window,
                duration: 0.2,
                options: .transitionCrossDissolve,
                animations: {},
                completion: nil
            )

        }
    }
}

extension SceneDelegate {
    enum NotificationMessage : NotificationCenterValue {
        case beginRegisterFlow(userId: String)
        case showTabBarController
        case resetToOnboarding
        case showLogin(userId: String)
        case logout(userId: String)
        case showLoginWithInactivityAlert(userId: String)
    }

    private func handleNotification(_ notification: NotificationMessage, animated: Bool) {
        switch notification {
        case .beginRegisterFlow(let userId):
            let choosePin = UIHostingController(rootView: ChoosePinCodeView(userId: userId))
            changeRootViewController(choosePin, animated: animated)
        case .showTabBarController:
            changeRootViewController(TabBarController(), animated: animated)
            enableActivityMonitor()
        case .resetToOnboarding:
            let onboarding = UIHostingController(rootView: OnboardingView())
            changeRootViewController(onboarding, animated: animated)
        case .showLogin(let userId):
            showLogin(userId: userId, logoutTrigger: false)
        case .logout(let userId):
            showLogin(userId: userId, logoutTrigger: true, showInactivityAlert: false)
        case .showLoginWithInactivityAlert(let userId):
            showLogin(userId: userId, logoutTrigger: true, showInactivityAlert: true)
        }
    }

    private func showLogin(userId: String, logoutTrigger: Bool, showInactivityAlert: Bool = false) {
        // Stop tracking, since we are logged out at this point
        timAuth.logout()
        activityMonitor.stopTracking()
        let loginView = UIHostingController(rootView: LoginView(userId: userId, presentedFromLogout: logoutTrigger))
        changeRootViewController(loginView, animated: true)
    }

    private func enableActivityMonitor() {
        activityMonitor.setupForRootViewControllerChanges()
        activityMonitor.startTracking()
    }
}
