// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import LocalAuthentication

struct LoginView: View {
    let userId: String

    @DI var logger: Logger
    @ObservedObject var viewModel: ViewModel
    @State var keyboardHeight: CGFloat = 0
    
    init(userId: String, presentedFromLogout: Bool) {
        self.viewModel = ViewModel(
            userId: userId,
            presentedFromLogout: presentedFromLogout
        )
        self.userId = userId
    }
    
    var body: some View {
        LoginBackgroundView()
            .overlay(
                VStack {
                    PINTitleView(
                        pinCodeLength: viewModel.pinCodeLength,
                        isBiometricLoginAvailable: viewModel.isBiometricLoginAvailable,
                        title: R.string.localizable.login_title(),
                        infoText: viewModel.loginText,
                        wrongCodeText: R.string.localizable.login_error_wrong_pin_code(),
                        pinCode: $viewModel.pinCode,
                        isLoading: $viewModel.isLoading,
                        wrongCode: $viewModel.wrongCode,
                        biometricLoginWasActivated: $viewModel.biometricLoginWasActivated,
                        shouldResetWrongPinCodeState: $viewModel.shouldResetWrongPinCodeState
                    )
                    .padding()
                    Button(R.string.localizable.login_forgot_password()) {
                        viewModel.forgotPassword = true
                    }
                    .font(Font(FontStyle.info.scaledFont()))
                    .foregroundColor(R.color.text.color())
                    .padding([.bottom])
                    if #available(iOS 14, *) {
                        // All good
                    } else {
                        Spacer(minLength: Metrics.Sizes.numberKeyboardHeight)
                    }
                }
                .padding()
            )
            .onAppear(perform: {
                let shouldAskForBioLogin: Bool = viewModel.isBiometricLoginAvailable &&
                    !viewModel.presentedFromLogout &&
                    !viewModel.biometricLoginWasActivated

                if shouldAskForBioLogin {
                    DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(300)) {
                        logger.info("Requesting auto login with biometric.")
                        viewModel.biometricLoginWasActivated = true
                    }
                }
            })
        // Multiple alerts have to be on different views to work in the same view
        VStack { }
            .alert(isPresented: $viewModel.sessionExpired, content: {
                Alert(
                    title: Text(R.string.localizable.common_alert_title_error()),
                    message: Text(R.string.localizable.login_error_refresh_token_expired()),
                    dismissButton: .default(Text(R.string.localizable.common_ok()), action: {
                        viewModel.resetToOnboarding()
                    })
                )
            })
        VStack { }
            .alert(isPresented: $viewModel.loginInvalidated, content: {
                Alert(
                    title: Text(R.string.localizable.common_alert_title_error()),
                    message: Text(R.string.localizable.login_error_key_locked()),
                    dismissButton: .default(Text(R.string.localizable.common_ok()), action: {
                        viewModel.resetToOnboarding()
                    })
                )
            })
        VStack { }
            .alert(isPresented: $viewModel.genericError, content: {
                Alert(
                    title: Text(R.string.localizable.common_alert_title_error()),
                    message: Text(R.string.localizable.common_alert_message_generic()),
                    dismissButton: .default(Text(R.string.localizable.common_ok()), action: {
                        viewModel.resetToOnboarding()
                    })
                )
            })
        VStack { }
            .sheet(isPresented: $viewModel.forgotPassword, content: {
                OnboardingConsentView()
            })
    }
}

struct LoginView_Previews: PreviewProvider {
    static var previews: some View {
        LoginView(userId: "<userId>", presentedFromLogout: false)
    }
}
