// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import LocalAuthentication
import TIM
import TIMEncryptedStorage
import Combine

extension LoginView {
    final class ViewModel : ObservableObject {
        @DI var logger: Logger
        @DI var timAuth: TIMAuth
        @DI var timStorage: TIMDataStorage
        @DI var localAuth: LocalAuthenticationContext
        private var cancelBag = CancelBag()

        let userId: String
        let presentedFromLogout: Bool

        let pinCodeLength: UInt = Metrics.Lengths.pinCode

        @Published var pinCode: String = ""
        @Published var isLoading: Bool = false
        @Published var wrongCode: Bool = false
        @Published var biometricLoginWasActivated: Bool = false
        @Published var shouldResetWrongPinCodeState: Bool = false

        @Published var loginInvalidated: Bool = false
        @Published var genericError: Bool = false
        @Published var sessionExpired: Bool = false

        @Published var forgotPassword: Bool = false

        var isBiometricLoginAvailable: Bool {
            localAuth.isBiometricAvailable &&
                timStorage.hasBiometricAccessForRefreshToken(userId: userId)
        }

        var loginText: String {
            let bioName = localAuth.biometricName
            if bioName.isEmpty {
                return R.string.localizable.login_text_no_biometrics_available()
            } else {
                return R.string.localizable.login_text(bioName)
            }
        }

        init(userId: String, presentedFromLogout: Bool) {
            self.userId = userId
            self.presentedFromLogout = presentedFromLogout
            self.observeChanges()
        }

        private func observeChanges() {
            $pinCode
                .sink { [weak self] (pinCode) in
                    guard let self = self else {
                        return
                    }
                    if pinCode.length == self.pinCodeLength, !self.isLoading {
                        self.loginWithPIN(pinCode)
                    }
                }
                .store(in: &cancelBag)

            $biometricLoginWasActivated
                .sink { [weak self] (activated) in
                    if activated {
                        self?.loginWithBio()
                    }
                }
                .store(in: &cancelBag)

            $shouldResetWrongPinCodeState
                .sink { [weak self] (reset) in
                    if reset {
                        self?.wrongCode = false
                        self?.pinCode = ""
                        self?.shouldResetWrongPinCodeState = false
                    }
                }
                .store(in: &cancelBag)
        }

        private func loginWithPIN(_ pin: String) {
            isLoading = true
            timAuth.loginWithPassword(userId: userId, password: pin, storeNewRefreshToken: true)
                .sink(
                    receiveCompletion: handleLoginCompletion,
                    receiveValue: {  _ in }
                )
                .store(in: &cancelBag)
        }

        private func loginWithBio() {
            isLoading = true
            timAuth.loginWithBiometricId(userId: userId, storeNewRefreshToken: true)
                .receive(on: DispatchQueue.main)
                .sink(
                    receiveCompletion: handleLoginCompletion,
                    receiveValue: {  _ in }
                )
                .store(in: &cancelBag)
        }
        
        private func handleLoginCompletion(completion: Subscribers.Completion<TIMError>) {
            switch completion {
            case .failure(let error):
                switch error {
                case .storage(let storageError):
                    if storageError.isWrongPassword() {
                        logger.debug("Wrong PIN")
                        wrongCode = true
                    } else if storageError.isKeyLocked() {
                        logger.debug("Wrong PIN was provided three times. The Login is invalidated.")
                        loginInvalidated = true
                    } else if case .failedToGetRefreshToken = storageError {
                        logger.info("Bio login cancelled or failed.")
                    } else {
                        logger.error("Failed to login with storage error: \(storageError.localizedDescription)")
                        genericError = true
                    }
                case .auth(let authError):
                    if case TIMAuthError.refreshTokenExpired = authError {
                        sessionExpired = true
                    } else {
                        logger.error("Failed to login: \(error.localizedDescription)")
                        genericError = true
                    }
                }
            case .finished:
                logger.info("Login completed!")
                NotificationCenter.default.post(
                    notification: NSNotification.Name.SceneDelegate.changeRootViewController,
                    value: SceneDelegate.NotificationMessage.showTabBarController
                )
            }
            isLoading = false
        }

        func resetToOnboarding() {
            NotificationCenter.default.post(
                notification: NSNotification.Name.SceneDelegate.changeRootViewController,
                value: SceneDelegate.NotificationMessage.resetToOnboarding
            )
        }

    }
}
