// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import MaternityData
import CMSServices

struct AppointmentDetailMeasurementsTable: View {
    @DI var dictionaryService: DictionaryService
    @Environment(\.viewController) private var viewControllerHolder: UIViewController?

    let title: String
    let observations: [Observation]
    var body: some View {
        VStack(alignment: .leading) {
            Text(title)
                .applyTextStyle(fontStyle: .h4)
            TwoColumnDataTable(
                rows: observations
                    .flatMap({ $0.values })
                    .map({ (ov: ObservationValue) -> [DataTableCell] in
                        let cellIdPrefix = ov.name ?? UUID().uuidString
                        return [
                            DataTableCell(
                                id: "\(cellIdPrefix)_name",
                                value: ov.name ?? "-",
                                type: cellType(for: ov),
                                fixedWidth: 140
                            ),
                            DataTableCell(
                                id: "\(cellIdPrefix)_value",
                                value: "\(ov.value ?? "-") \(ov.unit ?? "")",
                                type: .text,
                                fixedWidth: nil
                            )
                        ]
                    })
            )
            if let latestChange = observations.latestChange {
                Text(R.string.localizable.appointment_details_measurement_author(
                    latestChange.enteredBy,
                    AppointmentsView.dateAndTimeFormatter.string(from: latestChange.dateOfMeasurement)
                ))
                .applyTextStyle(fontStyle: .info, textColor: R.color.greyText.color())
            }
        }
    }

    private func cellType(for value: ObservationValue) -> DataTableCellType {
        let cellType: DataTableCellType
        if let name: String = value.name,
           let dictionaryText: String = dictionaryService.dictionary[name] {
            cellType = .button({ (cell: DataTableCell) in
                viewControllerHolder?.presentPartialSheet(
                    buttonColor: R.color.infoButton.color(),
                    content: {
                        InfoSheetContent(
                            title: name,
                            text: dictionaryText
                        )
                    })
            })
        } else {
            cellType = .boldText
        }
        return cellType
    }
}

#if DEBUG
struct AppointmentDetailMeasurementsTable_Previews: PreviewProvider {
    static var previews: some View {
        AppointmentDetailMeasurementsTable(
            title: "Test",
            observations:  loadPreviewCarePlans()
                .flatMap { $0.activities }
                .first!
                .observations
        )
    }
}
#endif
