// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import ChecklistData
import ErrorData
import Combine

struct ChecklistDetailItemRow: View {

    var item: ChecklistItem

    @State var isLoading: Bool = false
    @State var isExpanded: Bool = false
    @State private var cancelBag = CancelBag()

    var toggleItemEvent: (ChecklistItem) -> AnyPublisher<Void, ServiceError>

    private var canExpand: Bool {
        item.text != nil
    }

    private func toggleItem() {
        withAnimation {
            self.isLoading = true
        }
        toggleItemEvent(item)
        .receiveOnMain()
        .sink(
            receiveCompletion: { _ in
                withAnimation {
                    self.isLoading = false
                }
            },
            receiveValue: { _ in }
        )
        .store(in: &cancelBag)
    }

    var body: some View {
        BasicCardView {
            HStack(spacing: 0) {
                VStack(alignment: .leading) {
                    HStack {
                        Checkmark(
                            isChecked: Binding(
                                get: { item.value },
                                set: { _ in toggleItem() }
                            )
                        )
                        .opacity(isLoading ? 0.0 : 1.0)
                        .allowsHitTesting(!isLoading)
                        .overlay(
                            ActivityIndicatorView(
                                isAnimating: isLoading,
                                style: .medium,
                                hidesWhenStopped: true
                            )
                            .allowsHitTesting(false)
                        )
                        Text(item.title)
                            .applyTextStyle(fontStyle: .h4)
                            .padding([.leading])
                        Spacer()
                    }
                    if let infoText = item.text, isExpanded {
                        Text(infoText)
                            .applyTextStyle(fontStyle: .body)
                            .fixedSize(horizontal: false, vertical: true)
                            .padding([.top])
                    }
                }.padding()
                if canExpand {
                    DefaultDivider()
                        .padding(.leading)
                    VStack {
                        Button(action: {
                            withAnimation(.easeOut) {
                                isExpanded.toggle()
                            }
                        }, label: {
                            Image(uiImage: R.image.chevron_up()!)
                                .rotationEffect(Angle(degrees: isExpanded ? 0 : 180))
                        })
                        .padding()
                        if isExpanded {
                            Spacer()
                        }
                    }
                }
            }
        }
    }
}

#if DEBUG
struct ChecklistDetailItemRow_Previews: PreviewProvider {
    static var previews: some View {
        if #available(iOS 14.0, *) {
            LazyVStack {
                ForEach(ChecklistItem.previewList()) { item in
                    ChecklistDetailItemRow(
                        item: item,
                        isLoading: Bool.random(),
                        isExpanded: Bool.random(),
                        toggleItemEvent: { _ in
                            Just(Void())
                                .setFailureType(to: ServiceError.self)
                                .eraseToAnyPublisher()
                        }
                    )
                }
            }.padding()
        }
    }
}
#endif
