// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI
import LexiconData
import ChecklistData

struct KnowledgeView: View {
    @ObservedObject var viewModel: ViewModel
    @EnvironmentObject var environmentStore: EnvironmentStore
    @State private var scrollOffset: CGFloat = 0
    
    /// iOS 13 does not behave correctly for navigation bar height changes with the search bar. iOS 13 users will always see an inline nav bar to avoid this issue.
    var navigationBarDisplayModeForiOSVersion: NavigationBarItem.TitleDisplayMode {
        if #available(iOS 14, *) {
            return .automatic
        } else {
            return .inline
        }
    }
    
    private let searchBarContainerHeight: CGFloat = Metrics.Sizes.textFieldHeight +
        Metrics.Spacing.searchBarTopBottomMargin * 2
    
    init() {
        viewModel = ViewModel()
    }
    
    var body: some View {
        VStack {
            NavigationView {
                ZStack {
                    DataLoadingView(
                        state: viewModel.state,
                        animateStateChanges: true,
                        retryAction: {
                            if viewModel.isSearching {
                                self.viewModel.searchForCurrentQuery()
                            } else {
                                self.viewModel.fetchData()
                            }
                        },
                        content: {
                            ScrollViewOffset(
                                scrollOffset: $scrollOffset,
                                content: {
                                    if viewModel.isSearching && !viewModel.searchQuery.isEmpty {
                                        KnowledgeViewSearchResultsView(
                                            searchQuery: viewModel.searchQuery,
                                            searchResults: viewModel.searchResults
                                        )
                                        .padding()
                                    } else {
                                        CompatibleVStack {
                                            KnowledgeViewChecklistSection()
                                            KnowledgeViewCategoriesSection(categories: viewModel.categories)
                                        }
                                    }
                                })
                                .padding(.top, searchBarContainerHeight)
                        }
                    )
                    KnowledgeViewTopSearchView(
                        scrollOffset: scrollOffset,
                        height: searchBarContainerHeight,
                        searchQuery: $viewModel.searchQuery,
                        isFocused: $viewModel.searchFieldHasFocus
                    )
                }
                .navigationBarTitle(
                    Text(R.string.localizable.bottom_navigation_title_knowledge()),
                    displayMode: navigationBarDisplayModeForiOSVersion
                )
                .onReceive(viewModel.$checklists, perform: { checklists in
                    environmentStore.checklists = checklists
                })
            }
            .navigationViewStyle(StackNavigationViewStyle())
            if !viewModel.userContext.isAuthenticated {
                SUIButton<PrimaryButton>(R.string.localizable.onboarding_login_nemid_button()) {
                    NotificationCenter.default.post(
                        notification: Notification.Name.SceneDelegate.changeRootViewController,
                        value: SceneDelegate.NotificationMessage.resetToOnboarding
                    )
                }
                .frame(height: Metrics.Sizes.buttonHeight)
                .padding([.leading, .trailing, .bottom])
            }
        }
    }
}

struct KnowledgeView_Previews: PreviewProvider {
    static var previews: some View {
        KnowledgeView()
            .environmentObject(KnowledgeView.EnvironmentStore())
    }
}
