// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import SwiftUI
import LexiconData
import LexiconServices
import ChecklistData
import ChecklistServices
import ErrorData

extension KnowledgeView {
    final class ViewModel: ObservableObject {

        @DI private var checklistService: ChecklistService
        @DI private var lexiconService: LexiconService
        @DI var userContext: UserContextProtocol
        private var cancelBag: CancelBag = CancelBag()

        @Published var categories: [LexiconData.Category] = []
        @Published var state: DataLoadingState = .data
        @Published var checklists: [Checklist] = []

        @Published var searchResults: [LexiconData.Lexicon] = []
        @Published var searchQuery: String = ""
        @Published var searchFieldHasFocus: Bool = false
        var hasData: Bool {
            !categories.isEmpty || !checklists.isEmpty
        }
        var isSearching: Bool {
            !searchQuery.isEmpty || searchFieldHasFocus
        }

        init() {
            fetchData()
            listenForSearchEvents()
        }

        func fetchData() {
            state = .loading

            lexiconService.fetchCategories()
                .zipIf(userContext.isAuthenticated, checklistService.checklists(), fallbackOutput: [])
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] completion in
                        switch completion {
                        case .failure(let error):
                            self?.state = .error(error)
                        default: break
                        }
                    },
                    receiveValue: { [weak self] categories, checklists in
                        self?.categories = categories
                        self?.checklists = checklists
                        self?.updateStateForData()
                    })
                .store(in: &cancelBag)
        }

        private func listenForSearchEvents() {
            // Reduce duplicates and typing speed for direct searchQuery input
            let publisher = $searchQuery
                .removeDuplicates()
                .debounce(for: 0.2, scheduler: DispatchQueue.main)
                .eraseToAnyPublisher()

            search(queryPublisher: publisher)
        }

        /// Force searches for current query (designed for when in error state)
        func searchForCurrentQuery() {
            search(queryPublisher: Just(searchQuery).eraseToAnyPublisher())
        }

        private func updateStateForData() {
            self.state = hasData ? .data :
                .empty(
                    R.string.localizable.common_alert_title_error(),
                    R.string.localizable.common_alert_message_failed_to_load_data()
                )
        }

        /// Perform search based on a query publisher
        private func search(queryPublisher: AnyPublisher<String, Never>) {
            queryPublisher
                .handleEvents(receiveOutput: { (query) in
                    if query.isEmpty {
                        self.searchResults = [] //Reset if query is empty
                        self.updateStateForData()
                    }
                })
                .filter({ !$0.isEmpty })
                .setFailureType(to: ServiceError.self)
                .flatMap({ (query: String) -> AnyPublisher<[Lexicon], ServiceError> in
                    self.state = .loading
                    return self.lexiconService.fetchLexicons(
                        categories: [],
                        searchQuery: query
                    )
                    // Catch errors to avoid death of publisher ☠️
                    .catch({ [weak self] (error: ServiceError) -> Empty<[Lexicon], ServiceError> in
                        self?.state = .error(error)
                        return Empty()
                    })
                    .eraseToAnyPublisher()
                })
                .receiveOnMain()
                .replaceError(with: [])
                .sink(receiveValue: { [weak self] (results) in
                    self?.searchResults = results
                    if results.isEmpty {
                        self?.state = .empty(R.string.localizable.search_no_results_title(), R.string.localizable.search_no_results_description())
                    } else {
                        self?.state = .data
                    }
                })
                .store(in: &cancelBag)
        }
    }

    final class EnvironmentStore : ObservableObject {
        @Published var checklists: [ChecklistData.Checklist]

        init(checklists: [ChecklistData.Checklist] = []) {
            self.checklists = checklists
        }

        func updateChecklist(_ checklist: Checklist, checklistItem: ChecklistItem) {
            guard let listIndex = checklists.firstIndex(where: { $0.id == checklist.id }),
                  let itemIndex = checklists[listIndex].items.firstIndex(where: { $0.id == checklistItem.id }) else { return }

            let list = checklists[listIndex]
            var updatedList = checklists[listIndex].items
            updatedList[itemIndex] = checklistItem

            checklists[listIndex] = Checklist(id: list.id, title: list.title, text: list.text, items: updatedList)
        }
    }
}
