// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import Combine

struct PINCodeView: View {
    @DI var logger: Logger

    let pinCodeLength: UInt
    let isBiometricLoginAvailable: Bool

    var dotStates: [PINCodeDotState] {
        (0 ..< pinCodeLength).map { (i: UInt) -> PINCodeDotState in
            if wrongCode {
                return .error
            } else if i < pinCode.count || isLoading {
                return .filled
            } else if i == pinCode.count {
                return .active
            } else {
                return .empty
            }
        }
    }

    @Binding var pinCode: String
    @Binding var isLoading: Bool
    @Binding var wrongCode: Bool
    @Binding var biometricLoginWasActivated: Bool
    @Binding var shouldResetWrongPinCodeState: Bool

    var body: some View {
        SUIViewControllerWrapper<CustomNumberPadKeyboardViewController, SUITextFirstResponderWrapper>(content: {
            SUITextFirstResponderWrapper(
                text: $pinCode,
                becomeFirstResponder: true,
                validateInput: { (input: String) -> Bool in
                    // Reset when trying to type in more
                    if wrongCode {
                        shouldResetWrongPinCodeState = true
                    }
                    return NumberPadKeyboardKeyType.isValidInput(input)
                        && (pinCode.count + input.count) <= pinCodeLength
                },
                content: {
                    HStack(spacing: 30) {
                        ForEach(dotStates.indices, id: \.self) { (i: Int) in
                            PINCodeDot(state: dotStates[i])
                                .tag(i)
                                .modifier(
                                    VerticalSinusWaveAnimation(
                                        enabled: $isLoading
                                    )
                                )
                                .animation(
                                    Animation.easeInOut(duration: 1.0)
                                        .repeat(
                                            while: isLoading,
                                            autoreverses: true
                                        )
                                        .delay(0.15 * Double(i))
                                )
                        }
                    }
                    .modifier(ShakeAnimation(enabled: $wrongCode, shakes: 2))
                    .animation(.linear(duration: 0.3))
                })
        }, setupViewController: { (numberPadViewController: CustomNumberPadKeyboardViewController) in
            numberPadViewController.isBiometricKeyEnabled = isBiometricLoginAvailable
            numberPadViewController.keyEventActivated = { (key: NumberPadKeyboardKeyType) in
                switch key {
                case .bio:
                    biometricLoginWasActivated = true
                case .delete:
                    if wrongCode {
                        shouldResetWrongPinCodeState = true
                    }
                default:
                    break
                }
            }
        })
    }
}

struct PINCodeView_Previews: PreviewProvider {

    static var previews: some View {
        VStack {
            PINCodeView(
                pinCodeLength: 4,
                isBiometricLoginAvailable: true,
                pinCode: .constant(""),
                isLoading: .constant(false),
                wrongCode: .constant(false),
                biometricLoginWasActivated: .constant(false),
                shouldResetWrongPinCodeState: .constant(false)
            )
            PINCodeView(
                pinCodeLength: 4,
                isBiometricLoginAvailable: true,
                pinCode: .constant("12"),
                isLoading: .constant(false),
                wrongCode: .constant(false),
                biometricLoginWasActivated: .constant(false),
                shouldResetWrongPinCodeState: .constant(false)
            )
            PINCodeView(
                pinCodeLength: 4,
                isBiometricLoginAvailable: true,
                pinCode: .constant("1234"),
                isLoading: .constant(true),
                wrongCode: .constant(false),
                biometricLoginWasActivated: .constant(false),
                shouldResetWrongPinCodeState: .constant(false)
            )
            PINCodeView(
                pinCodeLength: 4,
                isBiometricLoginAvailable: true,
                pinCode: .constant("1234"),
                isLoading: .constant(true),
                wrongCode: .constant(true),
                biometricLoginWasActivated: .constant(false),
                shouldResetWrongPinCodeState: .constant(false)
            )
        }
    }
}
