// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct ExpandableText<Content : View>: View {
    var text: String
    var maxHeight: CGFloat
    @Binding var isExpanded: Bool
    var expandControl: () -> Content

    @State private var isTruncated: Bool = false
    private let fontStyle = FontStyle.body

    init(text: String, maxHeight: CGFloat, isExpanded: Binding<Bool>, @ViewBuilder expandControl: @escaping () -> Content) {
        self.text = text
        self.maxHeight = maxHeight
        self._isExpanded = isExpanded
        self.expandControl = expandControl
    }

    private func determineTruncation(_ geometry: GeometryProxy) {
        let total = text.boundingRect(
            with: CGSize(
                width: geometry.size.width,
                height: .greatestFiniteMagnitude
            ),
            options: .usesLineFragmentOrigin,
            attributes: [.font: fontStyle.scaledFont()],
            context: nil
        )

        if total.size.height > geometry.size.height {
            isTruncated = true
        }
    }

    var body: some View {
        VStack {
            Text(text)
                .applyTextStyle(fontStyle: fontStyle)
                .frame(maxHeight: isExpanded ? .infinity : maxHeight)
                .fixedSize(horizontal: false, vertical: true)
                .background(
                    GeometryReader { geometry in
                        Color.clear.onAppear {
                            self.determineTruncation(geometry)
                        }
                    }
                )

            DefaultDivider()

            if isTruncated || isExpanded {
                expandControl()
            }
        }
    }
}

struct ExpandableText_Previews: PreviewProvider {
    static var previews: some View {
        ExpandableText(
            text: String.loremIpsum(.large),
            maxHeight: 50,
            isExpanded: .constant(false),
            expandControl: {
                Button("Fold ind") { }
            }).padding()
    }
}
