// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI

final class SUITextFirstResponder: UIControl {
    @Binding var text: String
    let validateInput: ((String) -> Bool)
    override var canBecomeFirstResponder: Bool { true }

    init(text: Binding<String>, validateInput: ((String) -> Bool)? = nil) {
        self._text = text
        self.validateInput = validateInput ?? { _ in true }
        super.init(frame: .zero)
        self.backgroundColor = .clear
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}

extension SUITextFirstResponder: UIKeyInput {
    var hasText: Bool { !text.isEmpty }

    func insertText(_ text: String) {
        if self.validateInput(text) {
            self.text.append(contentsOf: text)
            self.sendActions(for: .editingChanged)
        }
    }

    func deleteBackward() {
        guard !text.isEmpty else { return }
        text.removeLast()
        self.sendActions(for: .editingChanged)
    }
}

struct SUITextFirstResponderWrapper<Content: View>: UIViewControllerRepresentable {
    @Binding var text: String
    var becomeFirstResponder: Bool
    let validateInput: ((String) -> Bool)?

    var content: () -> Content

    init(text: Binding<String>, becomeFirstResponder: Bool, validateInput: ((String) -> Bool)? = nil, content: @escaping () -> Content) {
        self._text = text
        self.becomeFirstResponder = becomeFirstResponder
        self.validateInput = validateInput
        self.content = content
    }

    func makeUIViewController(context: Context) -> SUIViewContainerViewController<SUITextFirstResponder> {
        let viewController = SUIViewContainerViewController(
            uiContainerView: SUITextFirstResponder(
                text: $text,
                validateInput: validateInput
            )
        )
        viewController.assignContentToRootView(content())
        return viewController
    }

    func updateUIViewController(_ uiViewController: SUIViewContainerViewController<SUITextFirstResponder>, context: Context) {
        uiViewController.assignContentToRootView(content())

        if self.becomeFirstResponder {
            uiViewController.uiContainerView.becomeFirstResponder()
        }
    }
}
