// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import UIKit
import WebKit

struct SUIWKWebView: UIViewRepresentable {
    var viewModel: ViewModel

    func makeUIView(context: Context) -> WKWebView {
        WKWebView()
    }

    func updateUIView(_ uiView: WKWebView, context: Context) {
        viewModel.failedToLoad = false
        viewModel.isLoading = true

        uiView.navigationDelegate = context.coordinator
        if uiView.url != viewModel.url { //Don't reload the same url multiple times
            let request = URLRequest(url: viewModel.url)
            uiView.load(request)
        }
    }

    func makeCoordinator() -> Coordinator {
        Coordinator(viewModel: viewModel)
    }

    /// Coordinator to handle NSObject requirement for navigation delegate.
    final class Coordinator : NSObject, WKNavigationDelegate {

        @DI var logger: Logger
        var viewModel: ViewModel

        init(viewModel: ViewModel) {
            self.viewModel = viewModel
        }

        func webView(_ webView: WKWebView, didCommit navigation: WKNavigation!) {
            logger.debug("Starting load of \(webView.url?.absoluteString ?? "<none>")")
        }

        func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
            viewModel.isLoading = false
            logger.debug("Finished loading \(webView.url?.absoluteString ?? "<none>")")
        }

        func webView(_ webView: WKWebView, didFail navigation: WKNavigation!, withError error: Error) {
            viewModel.failedToLoad = true
            viewModel.isLoading = false
            logger.debug("Failed to load \(webView.url?.absoluteString ?? "<none>")")
        }

        func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
            //Allow everyting for now
            decisionHandler(.allow)
        }
    }

    final class ViewModel: ObservableObject {
        @Published var isLoading: Bool = false
        @Published var failedToLoad: Bool = false
        let url: URL

        init(url: URL) {
            self.url = url
        }
    }
}
