// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

// Based on this article: https://fivestars.blog/swiftui/scrollview-offset.html

private struct OffsetPreferenceKey: PreferenceKey {
  static var defaultValue: CGFloat = .zero
  static func reduce(value: inout CGFloat, nextValue: () -> CGFloat) {}
}

struct ScrollViewOffset<Content: View>: View {
    private let frameLayer = "frameLayer"
    let content: Content
    @Binding var scrollOffset: CGFloat

    init(
        scrollOffset: Binding<CGFloat>,
        @ViewBuilder content: @escaping () -> Content) {
        self._scrollOffset = scrollOffset
        self.content = content()
    }

    var body: some View {
        ScrollView {
            offsetReader
            content
                .padding(.top, -8) // Reset offsetReader padding
        }
        .coordinateSpace(name: frameLayer)
        .onPreferenceChange(OffsetPreferenceKey.self, perform: { offset in
            let adjustOffset: CGFloat
            if #available(iOS 14, *) {
                adjustOffset = 0
            } else {
                // iOS 13 bug, where we have to take an extra offset into account for navigation bars ...
                let safeAreaTop = UIApplication.shared.windows.first(where: { $0.isKeyWindow })?.safeAreaInsets.top ?? 0
                adjustOffset = 96 + safeAreaTop
            }
            scrollOffset = offset - adjustOffset
        })
    }

    var offsetReader: some View {
        GeometryReader { proxy in
            Color.clear
                .preference(
                    key: OffsetPreferenceKey.self,
                    value: proxy.frame(in: .named(frameLayer)).minY
                )
        }
        .frame(height: 0)
        // this makes sure that the reader doesn't affect the content height
    }
}

struct ScrollViewOffset_Previews: PreviewProvider {
    static var previews: some View {
        ScrollViewOffset(
            scrollOffset: .constant(0),
            content: {
            Text("Content...")
        })
    }
}
