// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import TIM
import TIMEncryptedStorage
import Combine
import XCTest

let mockAccessToken = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJ1c2VySWQiLCJuYW1lIjoiSm9obiBEb2UiLCJpYXQiOjE1MTYyMzkwMjJ9.RT3_bUnfDHAnDNF_Hf1a8OMz7bPSCKr6MD5vyVNdVBA"
let mockRefreshToken = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJ1c2VySWQiLCJuYW1lIjoiSm9obiBEb2UgcmVmcmVzaCIsImlhdCI6MTUxNjIzOTAyMn0.29ibTzG_6nPhFeDhlOtqOpzA6ERlzGHtFQU-H986gMc"
let mockUserId = "userId"
let mockPassword = "1234"

final class TestMockTIMDataStorage: TIMDataStorage {
    var availableUserIds: Set<String> = [mockUserId]
    private var hasBioAccess: Bool = true

    init(hasBioAccess: Bool) {
        self.hasBioAccess = hasBioAccess
    }

    func hasRefreshToken(userId: String) -> Bool {
        availableUserIds.contains(userId)
    }

    func hasBiometricAccessForRefreshToken(userId: String) -> Bool {
        hasRefreshToken(userId: userId) && hasBioAccess
    }

    func disableBiometricAccessForRefreshToken(userId: String) {
        if availableUserIds.contains(userId) {
            hasBioAccess = false
        }
    }

    func clear(userId: String) {
        hasBioAccess = false
        availableUserIds.remove(userId)
    }

    func getStoredRefreshToken(userId: String, password: String, completion: @escaping (Result<JWT, TIMError>) -> Void) {
       notImplementedForTestYet()
    }

    func getStoredRefreshTokenViaBiometric(userId: String, completion: @escaping (Result<BiometricRefreshToken, TIMError>) -> Void) {
        notImplementedForTestYet()
    }

    func storeRefreshToken(_ refreshToken: JWT, withExistingPassword password: String, completion: @escaping (Result<Void, TIMError>) -> Void) {
        notImplementedForTestYet()
    }

    func storeRefreshToken(_ refreshToken: JWT, withNewPassword newPassword: String, completion: @escaping (Result<TIMESKeyCreationResult, TIMError>) -> Void) {
        notImplementedForTestYet()
    }

    func enableBiometricAccessForRefreshToken(password: String, userId: String, completion: @escaping (Result<Void, TIMError>) -> Void) {
        notImplementedForTestYet()
    }

    func enableBiometricAccessForRefreshToken(longSecret: String, userId: String) -> Result<Void, TIMError> {
        notImplementedForTestYet()
        return .failure(TIMError.storage(.failedToGetRefreshToken))
    }

    func getStoredRefreshToken(userId: String, password: String) -> Future<JWT, TIMError> {
        Future { promise in
            if self.availableUserIds.contains(userId) {
                if password == mockPassword {
                    promise(.success(mockAccessToken))
                } else {
                    promise(.failure(TIMError.storage(.encryptedStorageFailed(.keyServiceFailed(.badPassword)))))
                }
            } else {
                promise(.failure(TIMError.storage(.failedToGetRefreshToken)))
            }
        }
    }

    func getStoredRefreshTokenViaBiometric(userId: String) -> Future<BiometricRefreshToken, TIMError> {
        Future { promise in
            self.getStoredRefreshTokenViaBiometric(userId: userId, completion: promise)
        }
    }

    func storeRefreshToken(_ refreshToken: JWT, withExistingPassword password: String) -> Future<Void, TIMError> {
        Future { promise in
            guard let userId = refreshToken.userId else {
                promise(.failure(TIMError.storage(.failedToStoreRefreshToken)))
                return
            }
            if self.availableUserIds.contains(userId) && password == mockPassword {
                promise(.success(()))
            } else {
                promise(.failure(TIMError.storage(.encryptedStorageFailed(.keyServiceFailed(.badPassword)))))
            }
        }
    }

    func storeRefreshToken(_ refreshToken: JWT, withNewPassword newPassword: String) -> Future<TIMESKeyCreationResult, TIMError> {
        Future { promise in
            self.notImplementedForTestYet()
        }
    }

    func enableBiometricAccessForRefreshToken(password: String, userId: String) -> Future<Void, TIMError> {
        Future { promise in
            self.notImplementedForTestYet()
        }
    }

    private func notImplementedForTestYet() {
        XCTFail("TestMock implementation is not implemented for this function yet!")
    }
}
