// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
import DependencyInjection
import TIM
import Combine
@testable import MinGraviditet

class UserContextTests: XCTestCase {

    private var userContext = UserContext()
    @Inject private var timAuth: TIMAuth

    private var store: Set<AnyCancellable> = []

    override func setUpWithError() throws {
        continueAfterFailure = false

        Resolver.shared.allowOverride = true
        Resolver.shared.registerAsSingleton(TIMAuth.self, TestMockTIMAuth())
        Resolver.shared.registerAsSingleton(TIMDataStorage.self, TestMockTIMDataStorage(hasBioAccess: false))

        userContext.setHasHandledOnboardingFlow(false)
    }

    override func tearDownWithError() throws {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
    }

    func testHasHandledOnboardingFlow() throws {
        XCTAssertFalse(userContext.hasHandledOnboardingFlow)
        userContext.setHasHandledOnboardingFlow(true)
        XCTAssertTrue(userContext.hasHandledOnboardingFlow)
    }

    func testNotLoggedIn() {
        XCTAssertFalse(userContext.isAuthenticated)
        XCTAssertNil(userContext.userId)
    }

    func testLoginAndLogout() {
        let exp = XCTestExpectation(description: "Failed to login and logout")
        timAuth.loginWithPassword(userId: mockUserId, password: mockPassword, storeNewRefreshToken: true)
            .sink(
                receiveCompletion: { completion in
                    switch completion {
                    case .failure:
                        XCTFail("Failed to login!")
                    case .finished:
                        self.userContext.logout()
                        XCTAssertFalse(self.userContext.isAuthenticated)
                        XCTAssertTrue(self.userContext.hasUserRegistered)
                    }
                    exp.fulfill()
                },
                receiveValue: { (accessToken) in
                    XCTAssertTrue(self.userContext.isAuthenticated)
            })
            .store(in: &self.store)


        wait(for: [exp], timeout: 10.0)
    }

    func testResetUserData() {
        let exp = XCTestExpectation(description: "Failed to reset user data")
        timAuth.performOpenIDConnectLogin(presentingViewController: UIViewController())
            .sink(
                receiveCompletion: { completion in
                    switch completion {
                    case .failure:
                        XCTFail("Failed to login!")
                    case .finished:
                        self.userContext.resetAllUserData()
                        XCTAssertFalse(self.userContext.isAuthenticated)
                        XCTAssertFalse(self.userContext.hasUserRegistered)
                        XCTAssertNil(self.userContext.userId)
                    }
                    exp.fulfill()
                },
                receiveValue: { (accessToken) in
                    XCTAssertTrue(self.userContext.hasUserRegistered)
                    XCTAssertTrue(self.userContext.isAuthenticated)
            })
            .store(in: &self.store)


        wait(for: [exp], timeout: 10.0)
    }

}
